<?php
declare(strict_types=1);

namespace Strig\Cache;

use Psr\Cache\CacheItemPoolInterface;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Zend\Diactoros\Response\JsonResponse;

class Middleware implements MiddlewareInterface
{

    protected $cache;

    public function __construct(CacheItemPoolInterface $cacheItemPool)
    {
        $this->cache = $cacheItemPool;
    }


    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     * @throws \Psr\Cache\InvalidArgumentException
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $cacheKey = $this->getCacheKey($request);

        $cacheItem = $this->cache->getItem($cacheKey);
        if ($cacheItem->isHit()) {
            return new JsonResponse($cacheItem->get());
        }

        $data = $handler->handle($request);
        $data = json_decode($data->getBody()->getContents());

        $cacheItem->set($data);
        $this->cache->save($cacheItem);

        return $data;
    }

    protected function getCacheKey(ServerRequestInterface $request)
    {
        $path = $request->getServerParams()['PATH_INFO'];

        $key = sha1($path);
        $key = urlencode($path);

        return $key;
    }
}
